#!/usr/bin/ruby

#
require 'uri'
require 'digest/md5'

class MusicCategory
	@parent
	@path
	@name
	@groupId
	@targetId
	@childDir
	@childFile
	@depth
	
	def initialize root, base, targetexts, classId, index  = 0, depth = 0, parent = nil
		@path = base
		@name = (depth == 0 ) ? $RES['main.check.allMusic'] : File.basename( base )
		@childDir = Array.new
		@childFile = Array.new
		@groupId = classId
		@targetId = classId + '_' + index.to_s
		@depth = depth
		@parent = parent
		index = 0
    targetdir = root
    targetdir += '/' + base if base.size > 0

    Dir.foreach(targetdir){|file|
      begin
        if ( File::Stat.new(targetdir + '/' + file).directory? ) && ( file != '.' ) && ( file != '..' )
				  @childDir.push MusicCategory.new( root, base + '/' + file, targetexts, @targetId, index, depth + 1, self)
				  index += 1
			  else
				  @childFile.push base + '/' + file if targetexts.include? File.extname( file ).downcase
 			  end
      rescue 
      end
		}
		@childDir.sort!
		@childFile.sort!
	end

	def classId
		return @targetId
	end

	def groupId
		return @groupId
	end

	def path
		return @path
	end

	def name
		return @name
	end
	
	def depth
		return @depth
	end

	def has_dir
		return @childDir.size > 0
	end
	
	def has_file_r
		return true if has_file 
		@childDir.each{|elem|
			return true if elem.has_file_r
		}
		return false
	end

	def has_file
		return @childFile.size > 0
	end

	def each_dir
		@childDir.each{|elem|
			yield elem
		}
	end
	
	def each_file
		@childFile.each{|elem|
			yield elem
		}
	end

	def <=> target
		return @path <=> target.path if target.kind_of?(MusicCategory)
		return @path <=> target if target.kind_of?(String)
		return nil
	end
end

def writeCategoryBoxRoot source
	print "\n"
	writeCategoryBox source
end

def writeCategoryBox source
	return if source.has_file_r == false

  display = <<EndOfSource
    <table id=\"category@#{source.classId}\" style=\"width:100%;\" bgcolor=\"#ccffcc\"><tr>
    #{ "<td width=\"" + ( source.depth * 12 ).to_s + "\"></td>" if source.depth > 0 }
    #{ "<td width=\"1\"><input type=\"checkbox\" name=\"checkbox@#{source.classId}\" value=\"#{source.name}\" onclick=\"checkAllChild( '#{source.classId}')\"></td>" }
    <td>
    #{ "<a href=\"javascript:treeExpand('" + source.groupId + "','" + source.classId + "');\">" if source.depth > 0}
    #{source.name}
    #{ "</a>" if source.depth > 0 } 
    </td></tr></table>\n
	  <div class=\"#{source.groupId}\" id=\"child@#{source.classId}\"
	  #{"style=\"display:none;\"" if source.depth > 0}
	  #{">"}
EndOfSource
  print display

	if source.has_dir != false
		source.each_dir{|dir|
			writeCategoryBox dir
		}
	end
	if source.has_file != false
		print "<table id=\"file@#{source.classId}\" width=\"100%\" bgcolor=\"#f0fff0\">"
		index = 0
    display = ""
		source.each_file{|file|
			display += <<EndOfSource
        <tr>
			  <td width=\"#{(source.depth+1)*12 }\" bgcolor=\"#{(index % 2 == 0) ? "#f0fff0" : "#e0ffe0"}\" ></td>
			  <td width=\"1\" bgcolor=\"#{(index % 2 == 0) ? "#f0fff0" : "#e0ffe0"}\"><input type=\"checkbox\" name=\"file#{index}@#{source.classId}\", value=\"#{file}\"></td>
			  <td bgcolor=\"#{(index % 2 == 0) ? "#f0fff0" : "#e0ffe0"}\">
			  #{File.basename( file )}<br/>
		    </td></tr>
EndOfSource
			index = index + 1
		}
    print display
		print "</table>\n"
	end
	print "</div>"
end

class MusicPlayListManager
	@playlists
	def initialize base
		@playlists = Array.new
		Dir.foreach(base){ |file|
			if !File::Stat.new(base + '/' + file).directory? && File.extname(file) == ".playlist"
				playlist = parsefile base + '/' + file
				@playlists.push playlist
			end
		}
	end
	
	def parsefile file
		return MusicPlayList.new(file)
	end
	
	def sizePlayList
		return @playlists.size
	end
	
	def each
		@playlists.each{|elem|
			yield elem
		}
	end
	
	def indexOf index
		return @playlists[index]
	end
end

class MusicPlayList
	@file
	@title
	@defaultPlayStyle
	@musics
	
	def initialize file
		@musics = Array.new
		if file != nil && file.size > 0
			@file = File.basename( file )
		end
		begin
			File.open( file ) {|input|
				lines = input.readlines
				lines.each{|line|
					if !( line =~ /^#.*/ )
						elem = line.split( "=" )
						if elem[0] == "title"
							@title = elem[1]
						elsif elem[0] == "playstyle"
							@defaultPlayStyle = elem[1]
						elsif elem[0] == "item"
							@musics.push elem[1]
						end
					end
				}
			}
			@musics.sort!
		rescue
		end
	end
	
	def saveAs file
		@musics.sort!
		File.open( file, "w" ) { |output|
			@file = File.basename( file )
			output.write "title=#{@title}\n"
			output.write "playstyle=#{@defaultPlayStyle}\n"
			@musics.collect{|music|
				output.write "item=#{music}\n"
			}
		}
	end
	
	def file
		return @file
	end
	
	def title
		return @title
	end
	
	def setTitle newname
		@title = newname
	end
	
	def defaultPlayStyle
		return @defaultPlayStyle
	end
	
	def setDefaultPlayStyle newname
		@defaultPlayStyle = newname
	end
	
	def sizeMusic
		return @musics.size
	end
	
	def music index
		return @musics[index]
	end
	
	def each_music
		@musics.each{|music|
			yield music
		}
	end
	
	def setMusicAt index, file
		@musics[index] = file
	end
	
	def push file
		@musics.push file
	end
	
	def clear
		@music = Array.new
	end
	
	def removeAt index
		@music.delete_at index
	end
end

def writePlayListBox source
	print "<table border style=\"width:100%;\">"
	source.each{|playlist|
		print "<tr><td>"
		print "<a href=\"javascript:updateCurrentPlayList( '#{$CONF['cgi-bin'].to_s}/musicplaylist.rb', 'loginUser=#{$CGI.params['loginUser']}&loginUserMD5=#{$CGI.params['loginUserMD5']}&displayPlayListItem=*&playListFileName=/data/audio/playlist/#{playlist.file}' );\">#{playlist.title}</a>"
		print "</td></tr>"
	}
	print "</table>"
end

def displaySelectedPlaylistMusics item
	print "<table border style=\"width:100%;\">"
	item.each_music{|music|
		print "<tr><td>"
		print music
		print "</td></tr>"
	}
	print "</table>"
end
