#!/usr/bin/ruby
#
require 'uri'
require 'cgi'
require 'digest/md5'

class MillefeeMenuItem
	@requireJS
	@displayName
	@rhtmlBaseName
	@layoutFunction
	
	def initialize
		@requireJS = Array.new
	end
	
	def each_js
		@requireJS.each{|js|
			yield js
		}
	end
	
	def addRequireJS requireJS
		@requireJS.push requireJS
	end
	
	def displayName
		return @displayName
	end
	
	def setDisplayName displayName
		@displayName = displayName
	end
	
	def rhtmlBaseName
		return @rhtmlBaseName
	end
	
	def setRhtmlBaseName rhtmlBaseName
		@rhtmlBaseName = rhtmlBaseName
	end
	
	def layoutFunction
		return @layoutFunction
	end
	
	def setLayoutFunction layoutFunction
		@layoutFunction = layoutFunction
	end
end

class MillefeeMenuCategory
	@displayName
	@directoryName
	@items
	
	def initialize
		@items = Array.new
	end
	
	def each_item
		@items.each{|item|
			yield item
		}
	end
	
	def push item
		@items.push item
	end
	
	def displayName
		return @displayName
	end
	
	def setDisplayName displayName
		@displayName = displayName
	end

	def directoryName
		return @directoryName
	end
	
	def setDirectoryName directoryName
		@directoryName = directoryName
	end
end

def loadMillefeeMenuObject file
	categories = Array.new
	File.open( file ) { |file|
		category = nil
		item = nil
		current = nil
		file.readlines.each{|line|
			if line.strip == "category"
				category.push item if item != nil && category != nil
				categories.push category if category != nil
				category = MillefeeMenuCategory.new
				item = nil
				current = category
			elsif line.strip == "item"
				category.push item if item != nil && category != nil
				item = MillefeeMenuItem.new
				current = item
			else
				elem = line.split( "=" )
				if elem.size == 2
					if elem[0].strip == "displayname"
						current.setDisplayName elem[1].strip
					elsif elem[0].strip == "directoryName"
						current.setDirectoryName elem[1].strip
					elsif elem[0].strip == "rhtmlBaseName"
						current.setRhtmlBaseName elem[1].strip
					elsif elem[0].strip == "layoutFunction"
						current.setLayoutFunction elem[1].strip
					elsif elem[0].strip == "requireJS"
						current.addRequireJS elem[1].strip
					end
				end
			end
		}
		if current != nil
			if current.kind_of? MillefeeMenuItem
				category.push current if category != nil
			end
			categories.push category if category != nil
		end
	}
	return categories
end

# use for debug
def displayEnv
	print "<table>"
	ENV.each{|key,val|
		print "<tr><td>#{key}</td><td>#{val}</td></tr>"
	}
	print "</table>"
end

# use for debug
def displayCGI
	print "<table>"
	$CGI.params.each{|key,val|
		print "<tr><td>#{key}</td><td>#{val[0]}</td></tr>"
	}
	print "</table>"
end

def getBaseDirectory
	if ENV['PATH_TRANSLATED'] != nil
		base = File.dirname( ENV['PATH_TRANSLATED'] )
	elsif File.dirname( ENV['SCRIPT_FILENAME'] ) != nil
		require File.dirname( ENV['SCRIPT_FILENAME'] ) + '/../conf/millefee.conf.rb'
		base = $MUSIC_SERVER_RHTML_ROOT
	end
end

def createGlobalInstance base
	if $CGI == nil
		$CGI = CGI.new
	end
	if $CONF == nil
		config = base + '/conf/server.conf.rhtml'
		$CONF = loadConfig config
	end
	if $RES == nil
		resource = base + '/res/resource.conf'
		$RES = loadConfig resource
	end
	if $USERINFO == nil
		passwd = base + '/conf/password.conf.rhtml'
		$USERINFO = loadConfig passwd
	end
	ERuby.charset = $CONF['charset'] if $CONF['charset'] != nil
end

def displayHtmlHeader
	print "Content-type: text/html\n\n"
end

def displayExceptionMessage e
	print "#{e.message} in  #{e.class}<br/>"
	trace = e.backtrace
	print "from #{File.basename(trace.shift)} <br/>"
end

def validateLoginUser userName, password
	return false if userName == nil
	return false if password == nil
	return false if $USERINFO[userName.to_s] == nil
	return Digest::MD5.hexdigest(userName + ":" + password) == $USERINFO[userName.to_s]
end

def validateLoginUserMD5 userName, md5
	return false if userName == nil || md5 == nil
	return $USERINFO[userName.to_s] == md5.to_s
end

def createSupportedFileExtList type
	result = Array.new
	Dir.open($CONF['cgi-bin'] + '/' + type + '/filter'){|file|
		if !file.directory? && File.extname( file ) == ".rb"
			result.push "." + File.basename( file, ".rb" )
		end	
	}
end

def loadConfig config
	result = Hash.new	

	open( config ).each_line{|line|
		line.chomp!
		
		v = line.split('=')
		if v.size > 1 then
			result[v[0]] = v[1]
		elsif v.size == 1 then
			result[v[0]] = ''
		end
	}
	return result
end
